#!/usr/bin/env lua
require ("nixio.fs")
require ("uci")
require ("ubus")

json = require "json"

local port_index = {"2", "1"}
local sysfs_device_present = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s"

local sysfs_idVendor = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s/idVendor"
local sysfs_idProduct = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s/idProduct"

local sysfs_ecm = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s/1-%s:1.0/net/usb*"
local sysfs_qmi_wwan = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s/1-%s:1.4/net/wwan*"
local sysfs_qmi_wdm = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s/1-%s:1.4/usbmisc/cdc-wdm*"
local sysfs_at_port_ec2x = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s/1-%s:1.3/ttyUSB*"
local sysfs_at_port_ec200t = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s/1-%s:1.4/ttyUSB*"

local sysfs_modem_port_ec2x = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s/1-%s:1.2/ttyUSB*"
local sysfs_modem_port_ec200t = "/sys/devices/platform/1e1c0000.xhci/usb1/1-%s/1-%s:1.3/ttyUSB*"

local restart_network = false

local table_modem_type = {}
table_modem_type["2c7c:0125"] = "EC2x"
table_modem_type["2c7c:6026"] = "EC200T"

local ec2x_map = {}
ec2x_map["ecm"] = nil
ec2x_map["wwan"] = sysfs_qmi_wwan
ec2x_map["cdc"] = sysfs_qmi_wdm
ec2x_map["modem"] = sysfs_modem_port_ec2x
ec2x_map["at"] = sysfs_at_port_ec2x

local ec200t_map = {}
ec200t_map["ecm"] = sysfs_ecm
ec200t_map["wwan"] = nil
ec200t_map["cdc"] = nil
ec200t_map["modem"] = sysfs_modem_port_ec200t
ec200t_map["at"] = sysfs_at_port_ec200t

local default_map = {}
default_map["ecm"] = nil
default_map["wwan"] = nil
default_map["cdc"] = nil
default_map["modem"] = nil
default_map["at"] = nil

local table_modem_ports = {}
table_modem_ports["2c7c:0125"] = ec2x_map
table_modem_ports["2c7c:6026"] = ec200t_map
table_modem_ports["default"] = default_map


function trim(s)
	if s == nil then return "" end
   return (s:gsub("^%s*(.-)%s*$", "%1"))
end


function device_present(idx)
	if port_index[idx] then
		local device = string.format(sysfs_device_present, port_index[idx])
		local ret = nixio.fs.access(device)
		if ret then
			return ret
		else 
			return false
		end
	else
		return false
	end
end

function popen_str(cli_str)
	local out
	f = io.popen (cli_str)
	out = f:read('*all')
	f:close()
	return out
end


function cat_sysfs(path)
	local out
	return popen_str ("cat ".. path)
	
end

function find_id_vendor(idx)
	if port_index[idx] then
		return trim(cat_sysfs(string.format(sysfs_idVendor, port_index[idx])))
	else
		return ""
	end
end

function find_id_product(idx)
	if port_index[idx] then
		return trim(cat_sysfs(string.format(sysfs_idProduct, port_index[idx])))
	else
		return ""
	end
end

function vendorid_productid(idx)
	local idVendor, idProduct = find_id_vendor(idx), find_id_product(idx)
	local product_code = idVendor..":"..idProduct
	return product_code
end


function get_product_type(product_code)
		local product_type = table_modem_type[product_code] 
		if product_type == nil then
			return "Not Found"
		else
			return product_type
		end
end

function modem_type(idx)
	local device_present = device_present(idx)
	if device_present then
		local product_code = vendorid_productid(idx)
		local product_type = get_product_type(product_code)
		return product_code, product_type
	else
		return nil, "Not Found"
	end 
end

function format_port_path(path, idx)
	local path_formatted = string.format(path, port_index[idx], port_index[idx])
	return path_formatted
end

function find_port(path, idx)
	if path == nil then return nil end
	local ports, numbers = nixio.fs.glob(format_port_path(path, idx))
	for port in ports do
		return nixio.fs.basename(port)
	end
	return nil
end

function get_port_vars(vp_id)
	if vp_id == nil then
		return default_map
	else
		 local vars = table_modem_ports[vp_id]
		 if vars == nil then
		 	return default_map
		 else
		 	return vars
		 end
	end
end

function find_ports(vp_id, idx)
	local port_vars = get_port_vars(vp_id)
	local ecm, wwan, cdc, at
	local device_ports = {}
	device_ports["ecm"]  = find_port(port_vars["ecm"], idx)
	device_ports["wwan"]  = find_port(port_vars["wwan"], idx)
	device_ports["cdc"]  = find_port(port_vars["cdc"], idx)
	device_ports["modem"]  = find_port(port_vars["modem"], idx)	
	device_ports["at"]  = find_port(port_vars["at"], idx)
	return device_ports
end



function detect_modem_get_ports(idx)
	local vp_id, product = modem_type(idx)
	local ports = find_ports(vp_id, idx)
	ports["product"]=product
	return ports
end	


function build_modem_table()
	local modem_table = {}
	modem_table["LTE1"] = detect_modem_get_ports(1)
	modem_table["LTE2"] = detect_modem_get_ports(2)
	return modem_table
end

function section_exists(x, interface_name)
	if x:get("network", interface_name)==nil then
		return false
	else
		return true
	end
end

function create_cellular_section(x, interface_name, ports, i)
	metric = i * 25;
	x:set("network", interface_name, "interface")
	x:set("network", interface_name, "proto", "cellular")
	x:set("network", interface_name, "pdptype", "IP")
	x:set("network", interface_name, "proto", "cellular")
	x:set("network", interface_name, "ipv6", "auto")
	x:set("network", interface_name, "roaming", "2")
	x:set("network", interface_name, "metric", metric)
	x:set("network", interface_name, "device", "/dev/"..ports["modem"])
	x:commit("network")
	restart_network = true
end

function create_qmi_section(x, interface_name, ports, i)
	metric = i * 25;
	x:set("network", interface_name, "interface")
	x:set("network", interface_name, "proto", "qmi")
	x:set("network", interface_name, "ifname", ports["wwan"])
	x:set("network", interface_name, "metric", metric)
	x:set("network", interface_name, "device", "/dev/"..ports["cdc"])
	x:commit("network")
	restart_network = true
end

function create_or_update_interface(interface_name, ports, i)
	local x = uci.cursor()
	if ports["cdc"] ~= nil then
		print("Checking CDC")
		if not section_exists(x, interface_name) then
			print("Creating QMI section for ", interface_name)
			create_qmi_section(x, interface_name, ports, i)
		end
	end
	if ports["modem"] ~= nil then
		print("Checking Cellular")
		if not section_exists(x, interface_name) then
			print("Creating cellular section for ", interface_name)
			create_cellular_section(x, interface_name, ports, i)
		end
	end
end

function check_firewall_rule(modem_table)
	local x = uci.cursor()
	local rules_changed = false
	x:foreach("firewall", "zone", function(s)
		if s["name"]=="wan" then
			if s["network"] == nil or type(s["network"])=="table" or not s["network"]:match("LTE1") or not s["network"]:match("LTE2") then 
				print("adding LTE1 and LTE2 to zone")
				x:set("firewall", s[".name"], "network", "wan wan6 LTE1 LTE2")
				restart_network = true
				rules_changed = true
			else
				print("LTE1 and LTE2 already present in firewall zone")
			end
			--print("network setting is", s["network"])
		end
	end)
	if rules_changed then 
		x:commit("firewall")
	end
end

function restart_network_if_required()
	if restart_network == true then
		print("restarting network")
		local conn = ubus.connect()
		if not conn then
		    error("Failed to connect to ubusd")
		    return -1
		end
		conn:call("network", "restart", { })
	else
		print("Network restart not required")
	end
end

function create_or_update_network(modem_table)
	i = 1;
    for interface_name, ports in pairs(modem_table) do
    	create_or_update_interface(interface_name, ports, i)
		i=i+1;
    end
    check_firewall_rule(modem_table)
end

local modem_table = build_modem_table()
local modem_json = json.encode(modem_table)
print(modem_json)
nixio.fs.writefile("/var/xnet_modems.json", modem_json)
create_or_update_network(modem_table)
restart_network_if_required()
